// Main.h : Declaration of the _Main

#ifndef __MAIN_H_
#define __MAIN_H_

#include "resource.h"       // main symbols
#include <atlhost.h>
#include "EventHelper.h"

#import "..\..\controls\kSetEditor\kSetEditor.tlb" raw_interfaces_only, raw_native_types//, named_guids  
#import "..\..\controls\kSetButton\kSetButton.tlb" raw_interfaces_only, raw_native_types//, named_guids


template <class T>
class OkEvent : public _kEventHelper<IDC_OKBUTTON, T, KSETBUTTONLib::IkButtonEvents > 
{
public:
	STDMETHOD(Click)()
	{
		T* pT = static_cast<T*>(this);
		pT->ExitDialog();
		return S_OK;
	}
};

template <class T>
class AboutEvent : public _kEventHelper<IDC_ABOUTBUTTON, T, KSETBUTTONLib::IkButtonEvents > 
{
public:
	STDMETHOD(Click)()
	{
		T* pT = static_cast<T*>(this);
		pT->m_pAboutButton->AboutBox();
		return S_OK;
	}
};

template <class T>
class EditEvent : public _kEventHelper<IDC_KEDIT1, T, KSETEDITORLib::IkEditEvents >//, &KSETEDITORLib::IID_IkEditEvents>
{
	OLE_COLOR m_clrFore;
	OLE_COLOR m_clrBorder;
	bool m_bHover;
public:
	EditEvent() : m_bHover(false) {}
		
	STDMETHOD(Change)() {return S_OK;} 
		STDMETHOD(SetFocus)() 
		{			
			T* pT = static_cast<T*>(this);
			pT->m_pEdit->get_ForeColor(&m_clrFore);
			pT->m_pEdit->put_ForeColor(RGB(255, 0, 0));				
			return S_OK;
		}
		STDMETHOD(KillFocus)() 
		{
			T* pT = static_cast<T*>(this);
			pT->m_pEdit->put_ForeColor(m_clrFore);					
			return S_OK;
		}		
		STDMETHOD(Click)() {return S_OK;}
		STDMETHOD(DblClick)() {return S_OK;	}
		STDMETHOD(KeyDown)(short KeyCode, short Shift) {return S_OK;}
		STDMETHOD(KeyPress)(short KeyAscii) {return S_OK;}
		STDMETHOD(KeyUp)(short KeyCode, short Shift) 
		{
			if (KeyCode == VK_RETURN)
			{
				T* pT = static_cast<T*>(this);
				CComBSTR bstr(L"Enter");
				pT->m_pEdit->put_Text(bstr);
			}		
			return S_OK;
		}
		STDMETHOD(MouseDown)(short Button, short Shift, long X, long Y) {return S_OK;}
		STDMETHOD(MouseUp)(short Button, short Shift, long X, long Y) {return S_OK;}
		STDMETHOD(MouseMove)(short Button, short Shift, long X, long Y) 
		{
			if (!m_bHover)
			{
				m_bHover = true;
				T* pT = static_cast<T*>(this);
				pT->m_pEdit->get_BorderColor(&m_clrBorder);
				pT->m_pEdit->put_BorderColor(RGB(255, 255, 255));		
			}
			return S_OK;
		}
		STDMETHOD(MouseLeave)() 
		{
			m_bHover = false;
			T* pT = static_cast<T*>(this); 
			pT->m_pEdit->put_BorderColor(m_clrBorder);
			return S_OK;
		}
};

/////////////////////////////////////////////////////////////////////////////
// _Main
class _Main : 
	public CAxDialogImpl<_Main>,
	public IDispEventImpl<IDC_CLEARBUTTON, _Main>,
	public OkEvent< _Main >,	
	public AboutEvent< _Main >,
	public EditEvent< _Main > 
{
public:
	CComPtr<KSETBUTTONLib::IkButton> m_pAboutButton;
	CComPtr<KSETEDITORLib::IkEdit> m_pEdit;

public:
	_Main()
	{
	}
	~_Main()
	{
	}

	enum { IDD = IDD_MAIN };

BEGIN_MSG_MAP(_Main)
	MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)	
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	LRESULT OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CComPtr<IUnknown> pUnk;
		AtlAxGetControl(GetDlgItem(IDC_ABOUTBUTTON), &pUnk);
		pUnk.QueryInterface(&m_pAboutButton); 
		pUnk.Release();
		
		AtlAxGetControl(GetDlgItem(IDC_KEDIT1), &pUnk);
		pUnk.QueryInterface(&m_pEdit); 
		pUnk.Release();
		
		AboutEvent< _Main >::Advise();
		OkEvent< _Main >::Advise();
		EditEvent< _Main >::Advise();
		AtlAdviseSinkMap(this, true);
		return 1;  // Let the system set the focus
	}

	void ExitDialog()
	{	
		m_pAboutButton.Release();
		m_pEdit.Release();
		AboutEvent< _Main >::Unadvise();
		OkEvent< _Main >::Unadvise();
		EditEvent< _Main >::Unadvise();		
		AtlAdviseSinkMap(this, false);
		EndDialog(1);
	};	

	HRESULT __stdcall OnClickClearbutton()
	{	
		CComBSTR bstr(L"");
		m_pEdit->put_Text(bstr);
		return S_OK;
	}
public :

BEGIN_SINK_MAP(_Main)
	//Make sure the Event Handlers have __stdcall calling convention
	SINK_ENTRY(IDC_CLEARBUTTON, DISPID_CLICK, OnClickClearbutton)
END_SINK_MAP()

};

#endif //__MAIN_H_
